package node

import (
	"fmt"
	"net"
	"testing"
	"time"

	"git.autistici.org/ale/autoradio"
	"git.autistici.org/ale/autoradio/util"
)

type mockController struct {
	mounts     []*autoradio.Mount
	isMaster   bool
	masterAddr net.IP
}

func (m *mockController) Run() {
}

func (m *mockController) Update(conf *clusterConfig, isMaster bool, masterAddr net.IP) error {
	m.mounts = conf.ListMounts()
	m.isMaster = isMaster
	m.masterAddr = masterAddr
	return nil
}

func (m *mockController) GetStatus() *IcecastStatus {
	return &IcecastStatus{Up: true}
}

func startTestNodes(n int, etcd autoradio.EtcdClient) []*RadioNode {
	var nodes []*RadioNode

	for i := 0; i < n; i++ {
		node := NewRadioNode(
			fmt.Sprintf("node%d", i+1),
			[]net.IP{net.ParseIP(fmt.Sprintf("127.0.0.%d", i+1))},
			"eth0",
			1000,
			etcd)
		node.icecast = &mockController{}
		go node.Run()
		nodes = append(nodes, node)
	}
	time.Sleep(1100 * time.Millisecond)
	return nodes
}

func loadTestData(etcd autoradio.EtcdClient) {
	etcd.Set(autoradio.MountPrefix+"/test.ogg",
		`{"Name": "/test.ogg", "Username": "source1", "Password": "foo"}`,
		86400)
}

func countMasters(nodes []*RadioNode) int {
	var masters int
	for _, n := range nodes {
		if n.me.IsMaster() {
			masters++
		}
	}
	return masters
}

func TestRadioNode_MasterElection(t *testing.T) {
	etcd := util.NewTestEtcdClient()
	loadTestData(etcd)
	nodes := startTestNodes(3, etcd)

	// Shut down the nodes one by one, and verify that there is a
	// single master among the remaining ones.
	for i := 0; i < 3; i++ {
		if nm := countMasters(nodes[i:len(nodes)]); nm != 1 {
			t.Fatalf("@%d: masters=%d (expected 1)", i, nm)
		}
		nodes[i].Stop()
		time.Sleep(10 * time.Millisecond)
	}
}
